import os
import time
import logging

import pytest

from src.hlogger.hlogger import HLogger


TEST_LOG_FILE = "test_log.log"
NEW_TEST_LOG_FILE = "new_test_log.log"


@pytest.fixture
def cleanup():
    yield
    logging.shutdown()
    if os.path.exists(TEST_LOG_FILE):
        os.remove(TEST_LOG_FILE)
    if os.path.exists(NEW_TEST_LOG_FILE):
        os.remove(NEW_TEST_LOG_FILE)


def test_get_logger(cleanup):
    logger_name = "test_logger"
    logger = HLogger.get_logger(logger_name, TEST_LOG_FILE)

    assert isinstance(logger, logging.Logger)
    assert logger.name == logger_name


def test_set_level(cleanup):
    logger_name = "test_logger"
    logger = HLogger.get_logger(logger_name, TEST_LOG_FILE)

    # Custom level
    custom_level = logging.INFO

    # Set the log level to INFO
    HLogger.set_level(logger_name, custom_level)

    assert logger.level == custom_level
    for handler in logger.handlers:
        assert handler.level == custom_level


def test_set_formatter(cleanup):
    logger_name = "test_logger"
    logger = HLogger.get_logger(logger_name, TEST_LOG_FILE)

    # Custom formatter
    custom_formatter = logging.Formatter("%(message)s")

    # Set custom formatter
    HLogger.set_formatter(logger_name, custom_formatter)

    # Verify that all handlers have the custom formatter
    assert all(handler.formatter == custom_formatter for handler in logger.handlers)


def test_change_log_file(cleanup):
    logger_name = "test_logger"
    logger = HLogger.get_logger(logger_name, TEST_LOG_FILE)

    # Nothing to change just keep previous one
    HLogger.change_log_file(logger_name, TEST_LOG_FILE)

    # Simulate changing the log file
    HLogger.change_log_file(logger_name, NEW_TEST_LOG_FILE)

    # Verify that the new file handler was added to the logger
    assert any(
        isinstance(handler, logging.FileHandler)
        and handler.baseFilename.split(os.path.sep)[-1] == NEW_TEST_LOG_FILE
        for handler in logger.handlers
    )
    assert not any(
        isinstance(handler, logging.FileHandler)
        and handler.baseFilename.split(os.path.sep)[-1] == TEST_LOG_FILE
        for handler in logger.handlers
    )


def test_logging_to_file(cleanup):
    logger_name = "test_logger"

    # Set up logger
    logger = HLogger.get_logger(logger_name, TEST_LOG_FILE)

    # Log messages at different levels
    levels_to_test = [
        logging.DEBUG,
        logging.INFO,
        logging.WARNING,
        logging.ERROR,
        logging.CRITICAL,
    ]
    for level in levels_to_test:
        logger.log(level, f"Test log message at {logging.getLevelName(level)} level")

    # Wait for a moment to ensure the log is written to the file
    time.sleep(1)

    # Verify that the log file contains the logged message
    with open(TEST_LOG_FILE, "r") as file:
        log_content = file.read()
        for level in levels_to_test:
            assert (
                f"Test log message at {logging.getLevelName(level)} level"
                in log_content
            )



import os
import logging

import pytest

from src.hlogger import create_logger, get_logger, set_formatter


APP_LOG_FILE = "app_log.log"
NEW_APP_LOG_FILE = "new_app_log.log"


@pytest.fixture
def cleanup():
    yield
    logging.shutdown()
    if os.path.exists(APP_LOG_FILE):
        os.remove(APP_LOG_FILE)
    if os.path.exists(NEW_APP_LOG_FILE):
        os.remove(NEW_APP_LOG_FILE)


def test_create_logger(cleanup):
    # Set up
    logger_name = "app_logger"

    # Create logger and log a message
    logger = create_logger(logger_name, APP_LOG_FILE)
    logger.info("Application started")

    # Assertions
    assert isinstance(logger, logging.Logger)
    assert logger.name == logger_name

    # Check log content in the file
    with open(APP_LOG_FILE, "r") as file:
        log_content = file.read()
        assert f"Application started" in log_content


def test_get_logger(cleanup):
    # Set up
    logger_name1 = "app_logger1"
    logger_name2 = "app_logger2"

    # Create loggers with different names
    logger = create_logger(logger_name1, APP_LOG_FILE)
    logger = create_logger(logger_name2, NEW_APP_LOG_FILE)

    # Retrieve a logger and log a message
    logger = get_logger(logger_name1)
    logger.info("Application started")

    # Assertions
    assert isinstance(logger, logging.Logger)
    assert logger.name == logger_name1 and logger.name != logger_name2

    # Check log content in the file
    with open(APP_LOG_FILE, "r") as file:
        log_content = file.read()
        assert f"Application started" in log_content
    with open(NEW_APP_LOG_FILE, "r") as file:
        log_content = file.read()
        assert f"Application started" not in log_content


def test_set_formatter(cleanup):
    # Set up
    logger_name = "app_logger"
    custom_formatter = logging.Formatter("%(message)s")

    # Create logger
    logger = create_logger(logger_name, APP_LOG_FILE)

    # Log a message before setting the formatter
    logger.info("Message before setting formatter")

    # Set custom formatter and log another message
    set_formatter(logger_name, custom_formatter)
    logger.info("Message after setting formatter")

    # Verify log content with and without custom format
    with open(APP_LOG_FILE, "r") as file:
        log_content = file.read()

        # Assert message before setting formatter doesn't have custom format
        assert log_content.split("\n")[0] != ("Message before setting formatter")

        # Assert message after setting formatter has custom format
        assert log_content.split("\n")[1] == ("Message after setting formatter")


def test_change_logger(cleanup):
    # Set up
    logger_name = "app_logger1"

    # Create logger with the first log file, then change to a new log file
    logger = create_logger(logger_name, APP_LOG_FILE)
    logger = create_logger(logger_name, NEW_APP_LOG_FILE)

    # Log a message to the new log file
    logger.info("Application started")

    # Assertions
    assert isinstance(logger, logging.Logger)
    assert logger.name == logger_name

    # Check log content in the new file
    with open(NEW_APP_LOG_FILE, "r") as file:
        log_content = file.read()
        assert f"Application started" in log_content



"""APIs for user convenience."""

from typing import Optional, List

import logging

from .hlogger import HLogger


# List to store all created logger instances.
loggers: List[logging.Logger] = []


def create_logger(
    name: str,
    file_name: str,
    max_file_size: int = 10485760,
    backup_count: int = 10,
    level: int = logging.DEBUG,
) -> logging.Logger:
    """Create a logger and set it as the global logger if not already present in loggers.

    Args:
        name (str): Name of the logger
        file_name (str): Name of the logging file
        max_file_size (int): Maximum size of the logging file
        backup_count (int): Number of backup files
        level (int): Log level for the logger and handlers
    """
    global loggers

    new_logger = HLogger.get_logger(name, file_name, max_file_size, backup_count, level)
    if new_logger not in loggers:
        loggers.append(new_logger)
    return new_logger


def get_logger(name: str) -> Optional[logging.Logger]:
    """Get the global logger from the one with the specified name from loggers.

    Args:
        name (str): Name of the logger to set as the global logger

    Returns:
        The logger instance with the specified name, or None if not found.
    """
    global loggers

    for l in loggers:
        if l.name == name:
            return l


def set_formatter(name: str, formatter: logging.Formatter) -> None:
    """Set the formatter for all handlers of the logger with the specified name.

    Args:
        name (str): Name of the logger
        formatter (logging.Formatter): Formatter to use
    """
    HLogger.set_formatter(name, formatter)



"""Utility module for logging."""

import logging
from logging.handlers import RotatingFileHandler
from threading import Lock


class HLogger:
    """Utility class for logging to both file and stream."""

    _lock = Lock()

    @classmethod
    def get_logger(
        cls,
        name: str,
        file_name: str,
        max_file_size: int = 10485760,
        backup_count: int = 10,
        level: int = logging.DEBUG,
    ) -> logging.Logger:
        """Get or create a logger with file and stream handlers.

        Args:
            name (str): Name of the logger
            file_name (str): Name of the logging file
            max_file_size (int): Maximum size of the logging file
            backup_count (int): Number of backup files
            level (int): Log level for the logger and handlers

        Returns:
            Configured logger
        """
        with cls._lock:
            logger = logging.getLogger(name)
            logger.setLevel(level)

            # File Handler
            cls._add_file_handler(logger, file_name, max_file_size, backup_count, level)

            # Stream Handler
            cls._add_stream_handler(logger, level)

            return logger

    @classmethod
    def set_level(cls, name: str, level: int) -> None:
        """Set the log level of the logger and its handlers.

        Args:
            name (str): Name of the logger
            level (int): Log level to set
        """
        with cls._lock:
            logger = logging.getLogger(name)
            for handler in logger.handlers:
                handler.setLevel(level)
            logger.setLevel(level)

    @classmethod
    def set_formatter(cls, name: str, formatter: logging.Formatter) -> None:
        """Set the formatter for all handlers of the logger.

        Args:
            name (str): Name of the logger
            formatter (logging.Formatter): Formatter to use
        """
        with cls._lock:
            logger = logging.getLogger(name)
            for handler in logger.handlers:
                handler.setFormatter(formatter)

    @classmethod
    def change_log_file(
        cls,
        name: str,
        file_name: str,
        max_file_size: int = 10485760,
        backup_count: int = 10,
        level: int = logging.DEBUG,
    ) -> None:
        """Change the log file for the logger.

        Args:
            name (str): Name of the logger
            file_name (str): New name of the logging file
            max_file_size (int): Maximum size of the new logging file
            backup_count (int): Number of backup files for the new file
            level (int): Log level for the logger and handlers
        """
        with cls._lock:
            logger = logging.getLogger(name)
            for handler in logger.handlers:
                if isinstance(handler, RotatingFileHandler):
                    if handler.baseFilename.split("/")[-1] == file_name:
                        return

                    # New File Handler
                    new_file_handler = RotatingFileHandler(
                        filename=file_name,
                        maxBytes=max_file_size,
                        backupCount=backup_count,
                    )
                    new_file_handler.setFormatter(
                        logging.Formatter(
                            "[%(asctime)s %(name)10s - %(levelname)8s] %(message)s"
                        )
                    )
                    new_file_handler.setLevel(level)

                    logger.removeHandler(handler)
                    logger.addHandler(new_file_handler)

    @classmethod
    def _add_file_handler(
        cls,
        logger: logging.Logger,
        file_name: str,
        max_file_size: int = 10485760,
        backup_count: int = 10,
        level: int = logging.DEBUG,
    ) -> None:
        """Add or update a rotating file handler to the logger.

        Args:
            logger (logging.Logger): Logger to which the handler will be added or updated
            file_name (str): Name of the logging file
            max_file_size (int): Maximum size of the logging file
            backup_count (int): Number of backup files
            level (int): Log level for the handler
        """
        for handler in logger.handlers:
            if isinstance(handler, RotatingFileHandler):
                if handler.baseFilename.split("/")[-1] != file_name:
                    logger.removeHandler(handler)
                    break
                return
        file_handler = RotatingFileHandler(
            filename=file_name, maxBytes=max_file_size, backupCount=backup_count
        )
        file_handler.setFormatter(
            logging.Formatter("[%(asctime)s %(name)10s - %(levelname)8s] %(message)s")
        )
        file_handler.setLevel(level)
        logger.addHandler(file_handler)

    @classmethod
    def _add_stream_handler(
        cls,
        logger: logging.Logger,
        level: int = logging.DEBUG,
    ) -> None:
        """Add a stream handler to the logger if not already present.

        Args:
            logger (logging.Logger): Logger to which the handler will be added
            level (int): Log level for the handler
        """
        if not any(
            type(handler) == logging.StreamHandler for handler in logger.handlers
        ):
            stream_handler = logging.StreamHandler()
            stream_handler.setFormatter(
                logging.Formatter(
                    "[%(asctime)s %(name)10s - %(levelname)8s] %(message)s"
                )
            )
            stream_handler.setLevel(level)
            logger.addHandler(stream_handler)
